/*
        This sample code is licensed under the
        International License Agreement for
        non-warranted IBM Software and is
        provided "as-is" with no formal support.

        IBM COPYRIGHT INFORMATION:
        __________________________

        International Business Machines Corporation 1997, 2005
        Licensed program - Property of IBM . All rights reserved.
        IBM and WebSphere are trademarks of International Business
        Machines Corporation in the United States and/or other countries.
        Java is a trademark of Sun Microsystems, Incorporated.
        U.S. Government Users Restricted Rights - Use, duplication, or
        disclosure restricted by GSA ADP. Schedule Contract with IBM Corp.

        LIMIT OF LIABILITY:
        ___________________

        License to the contained APAR fixes is conditioned upon
        the recipient holding a license to the prerequisite
        product.  The license terms and conditions of the APAR
        fixes are the same as those for the prerequisite product.
        Please see the product license for specifics.
*/
package com.ibm.wps.portlets.hodwps5;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

import org.apache.jetspeed.portlet.*;
import org.apache.jetspeed.portlet.event.*;

import com.ibm.wps.portletservice.credentialvault.*;
import com.ibm.wps.portletservice.credentialvault.credentials.UserPasswordPassiveCredential;
import com.ibm.wps.util.ObjectID;

/**
 *
 * PasswordEditorPortlet
 * 
 */
public class PasswordEditorPortlet
    extends PortletAdapter
    implements ActionListener {

    // JSP file name to be rendered on the view mode
    public static final String VIEW_JSP = "/passwordeditor/jsp/PasswordEditorPortletView.";
    // JSP file name to be rendered on the view mode
    public static final String VIEW2_JSP = "/passwordeditor/jsp/PasswordEditorPortletView2.";
    // JSP file name to be rendered on the edit mode
    public static final String EDIT_JSP = "/passwordeditor/jsp/PasswordEditorPortletEdit.";
    // Bean name for the view mode request
    public static final String VIEW_BEAN = "com.ibm.wps.portlets.hodwps5.PasswordEditorPortletViewBean";
    // Bean name for the edit mode request
    public static final String EDIT_BEAN = "com.ibm.wps.portlets.hodwps5.PasswordEditorPortletEditBean";
    // Bean name for the portlet session
    public static final String SESSION_BEAN = "com.ibm.wps.portlets.hodwps5.PasswordEditorPortletSessionBean";
    // Action name for the userID/password entry form
    public static final String USER_ACTION = "com.ibm.wps.portlets.hodwps5.PasswordEditorPortletUserAction";
    // Parameter name for the userID text input
    public static final String USERID = "com.ibm.wps.portlets.hodwps5.PasswordEditorPortletUserId";
    // Parameter name for the password text input
    public static final String PASSWORD = "com.ibm.wps.portlets.hodwps5.PasswordEditorPortletPassword";
    // Parameter name for general text input
    public static final String PASSWORD_AGAIN = "com.ibm.wps.portlets.hodwps5.PasswordEditorPortletPasswordAgain";
    // Parameter name for general text input
    public static final String TEXT = "com.ibm.wps.portlets.hodwps5.PasswordEditorPortletText";
    // Parameter name for general submit button
    public static final String SUBMIT = "com.ibm.wps.portlets.hodwps5.PasswordEditorPortletSubmit";
    // Parameter name for general cancel button
    public static final String CANCEL = "com.ibm.wps.portlets.hodwps5.PasswordEditorPortletCancel";

    public static final String SLOTID = "com.ibm.wps.portlets.hodwps5.PasswordEditorPortletSlotId";

    /**
     * @see org.apache.jetspeed.portlet.Portlet#init(PortletConfig)
     */
    public void init(PortletConfig portletConfig) throws UnavailableException {
        super.init(portletConfig);
    }

    /**
     * @see org.apache.jetspeed.portlet.Portlet#initConcrete(PortletSettings)
     */
    public void initConcrete(PortletSettings settings)
        throws UnavailableException {
        super.initConcrete(settings);
        // Initialize the secret manager
        try {
            PasswordEditorPortletSecretManager.init(getPortletConfig());
        } catch (Exception e) {
            if (getPortletLog().isErrorEnabled())
                getPortletLog().error(
                    "Error on PasswordEditorPortletSecretManager.init(): "
                        + e.getMessage());
        }
    }

    /**
     * @see org.apache.jetspeed.portlet.PortletAdapter#doView(PortletRequest, PortletResponse)
     */
    public void doView(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        // Check if portlet session exists
        PasswordEditorPortletSessionBean sessionBean = getSessionBean(request);
        if (sessionBean == null) {
            response.getWriter().println("<b>NO PORTLET SESSION YET</b>");
            return;
        }
        
        /*
         * Re-display Edit JSP when a password error was found.
         */
        String passwordError = (String)request.getAttribute("passwordError");
        if (passwordError != null) {
            request.removeAttribute("passwordError");
            if (Boolean.valueOf(passwordError).booleanValue()) {
                // Invoke the JSP to render             
                getPortletConfig().getContext().include(
                    VIEW2_JSP + getJspExtension(request),
                    request,
                    response);
                return;                
            }
        }

        // Make a view mode bean
        PasswordEditorPortletViewBean viewBean =
            new PasswordEditorPortletViewBean();
        request.setAttribute(VIEW_BEAN, viewBean);

        // Retrieve user credentials
        StringBuffer userId = new StringBuffer("");
        StringBuffer password = new StringBuffer("");
        try {
            PasswordEditorPortletSecretManager.getCredential(
                request,
                sessionBean,
                userId,
                password);
        } catch (Exception e) {
            if (getPortletLog().isWarnEnabled())
                getPortletLog().warn(
                    "Warning on PasswordEditorPortletSecretManager.getCredential(): "
                        + e.getMessage());
        }
        // Portlet should use userId/password to log in to the backend systems on behalf of the user.
        // Show curent userId/password on the portal page at this time.

        // Set current userId/password in the view mode bean
        viewBean.setUserId(userId.toString());
        viewBean.setPassword(password.toString());

        // Retrieve all accessible credentials
        retrieveCredentials(request, viewBean);

        // Invoke the JSP to render        		
        getPortletConfig().getContext().include(
            VIEW_JSP + getJspExtension(request),
            request,
            response);
    }

    /**
     * @see org.apache.jetspeed.portlet.PortletAdapter#doEdit(PortletRequest, PortletResponse)
     */
    public void doEdit(PortletRequest request, PortletResponse response)
        throws PortletException, IOException {
        String slotId = request.getParameter(SLOTID);
        boolean passwordAgain =
            Boolean
                .valueOf(request.getParameter(PASSWORD_AGAIN))
                .booleanValue();
        
        // If no slot IDs are found, return to the View mode
        if (slotId == null || slotId.length() == 0) {
            String formName = response.encodeNamespace("HiddenForm");
            response.getWriter().println(
                "<FORM NAME= '"
                    + formName
                    + "' ACTION='"
                    + response.createReturnURI().toString()
                    + "' METHOD='POST'><INPUT TYPE='HIDDEN' NAME='DUMMY' VALUE=''></FORM>\n"
                    + "<script type='text/javascript' language='Javascript'>\n"
                    + "document.forms['" + formName + "'].submit();\n"
                    + "</script>");
            return;
        }

        // Check if portlet session exists
        PasswordEditorPortletSessionBean sessionBean = getSessionBean(request);
        if (sessionBean == null) {
            response.getWriter().println("<b>NO PORTLET SESSION YET</b>");
            return;
        }

        // Make an edit mode bean
        PasswordEditorPortletEditBean editBean =
            new PasswordEditorPortletEditBean();
        request.setAttribute(EDIT_BEAN, editBean);

        if (PasswordEditorPortletSecretManager.isWritable(sessionBean)) {
            // Set actionURI in the edit mode bean
            PortletURI userActionURI = response.createReturnURI();
            userActionURI.addAction(USER_ACTION);
            editBean.setUserActionURI(userActionURI.toString());
            // Retrieve user credentials
            StringBuffer userId = new StringBuffer("");
            StringBuffer password = new StringBuffer("");
            // Set slot ID and secret type for retrieving the secret 
            sessionBean.setVaultSlotName(slotId);
            sessionBean.setSecretType(
                PasswordEditorPortletSecretManager.SECRET_ADMINISTRATIVE_SLOT);
            
            try {
                PasswordEditorPortletSecretManager.getCredential(
                    request,
                    sessionBean,
                    userId,
                    password);
            } catch (Exception e) {
                if (getPortletLog().isWarnEnabled())
                    getPortletLog().warn(
                        "Warning on PasswordEditorPortletSecretManager.getCredential(): "
                            + e.getMessage());
            }
            // Set current userId/password in the edit mode bean
            editBean.setUserId(userId.toString());
            editBean.setPassword(password.toString());

            // Set slot Id
            editBean.setSlotId(slotId);
            
            // Set Password Again? flag
            editBean.setPasswordAgain(passwordAgain);
        }

        // Invoke the JSP to render        		
        getPortletConfig().getContext().include(
            EDIT_JSP + getJspExtension(request),
            request,
            response);
    }

    /**
     * @see org.apache.jetspeed.portlet.event.ActionListener#actionPerformed(ActionEvent)
     */
    public void actionPerformed(ActionEvent event) throws PortletException {
        if (getPortletLog().isDebugEnabled())
            getPortletLog().debug("ActionListener - actionPerformed called");
        // ActionEvent handler
        String actionString = event.getActionString();
        // Add action string handler here
        PortletRequest request = event.getRequest();
        PasswordEditorPortletSessionBean sessionBean = getSessionBean(request);
        if (USER_ACTION.equals(actionString)) {
            if (request.getParameter(SUBMIT) != null) {
                // Set userId/password text in the credential vault
                String userID = request.getParameter(USERID);
                String password = request.getParameter(PASSWORD);
                String passwordAgain = request.getParameter(PASSWORD_AGAIN);
                String slotID = request.getParameter(SLOTID);
                if (getPortletLog().isInfoEnabled())
                    getPortletLog().info("actionPerformed:  Slot ID=" + slotID);
                // Check if two password inputs match
                if (password == null 
                    || passwordAgain == null
                    || !password.equals(passwordAgain)) {
                    request.setAttribute("passwordError", "true");
                    return;
                }
                // Save only if both parameters are set
                if (userID != null
                    && password != null
                    && !userID.trim().equals("")
                    && !password.trim().equals("")) {
                    try {
                        PasswordEditorPortletSecretManager.setCredential(
                            request,
                            slotID,
                            userID,
                            password);
                    } catch (Exception e) {
                        if (getPortletLog().isErrorEnabled())
                            getPortletLog().error(
                                "Error on PasswordEditorPortletSecretManager.setCredential(): "
                                    + e.getMessage());
                    }
                }
            }
        }
    }

    /**
     * Get SessionBean.
     * 
     * @param request PortletRequest
     * @return PasswordEditorPortletSessionBean
     */
    private PasswordEditorPortletSessionBean getSessionBean(PortletRequest request) {
        PortletSession session = request.getPortletSession();
        PasswordEditorPortletSessionBean sessionBean =
            (PasswordEditorPortletSessionBean)session.getAttribute(
                SESSION_BEAN);
        if (sessionBean == null) {
            sessionBean = new PasswordEditorPortletSessionBean();
            session.setAttribute(SESSION_BEAN, sessionBean);
            // Retrieve secret type and vault slot id from the portlet settings
            String secretTypeValue =
                request.getPortletSettings().getAttribute("SecretType");
            if (secretTypeValue != null) {
                try {
                    sessionBean.setSecretType(
                        Integer.parseInt(secretTypeValue));
                } catch (NumberFormatException nfe) {
                    if (getPortletLog().isErrorEnabled())
                        getPortletLog().error(
                            "Invalid secret type: " + nfe.getMessage());
                }
            }
            sessionBean.setVaultSlotName(
                request.getPortletSettings().getAttribute("VaultSlotName"));
        }
        return sessionBean;
    }

    /**
     * Returns the file extension for the JSP file
     * 
     * @param request PortletRequest
     * @return JSP extension 
     */
    private static String getJspExtension(PortletRequest request) {
        return "jsp";
    }

    /**
     * Retrieve all accessible userid/password type credentials.
     * 
     * @param request
     * @param viewBean
     */
    private void retrieveCredentials(
        PortletRequest request,
        PasswordEditorPortletViewBean viewBean) {

        PortletContext context = getPortletConfig().getContext();
        CredentialVaultService cvs = null;

        try {
            cvs =
                (CredentialVaultService)context.getService(
                    CredentialVaultService.class);
        } catch (Exception e) {
            if (getPortletLog().isErrorEnabled())
                getPortletLog().error("could not access credential vault");
            return;
        }

        Iterator it;
        try {
            it = cvs.getAccessibleSlots(request);
        } catch (Exception e) {
            if (getPortletLog().isErrorEnabled())
                getPortletLog().error(
                    "getAccessibleSlots() failed: " + e.getLocalizedMessage());
            return;
        }

        List vaultSegments;
        try {
            vaultSegments = cvs.getAllVaultSegments();
        } catch (Exception e) {
            if (getPortletLog().isErrorEnabled())
                getPortletLog().error(
                    "getAllVaultSegments() failed: " + e.getLocalizedMessage());
            return;
        }

        Hashtable vaultSegmentsHash = new Hashtable();
        int size = vaultSegments.size();
        for (int i = 0; i < size; ++i) {
            VaultSegmentConfig vsc = (VaultSegmentConfig)vaultSegments.get(i);
            ObjectID oid = vsc.getObjectID();
            String name = vsc.getName();
            vaultSegmentsHash.put(oid, name);
        }

        Vector adminCredentials = new Vector();

        while (it.hasNext()) {
            CredentialSlotConfig config = (CredentialSlotConfig)it.next();

            String id = config.getSlotId();

            if (id == null)
                continue;

            String resourceName = config.getResourceName();
            String segmentName =
                (String)vaultSegmentsHash.get(config.getSegmentObjectID());

            // 1. If this resource does not store userid/password, or
            // 2. If this is a system slot, or
            // 3. If this is a shared slot, 
            // => let's skip it
            if (config.getSecretType()
                != CredentialVaultService
                    .SECRET_TYPE_USERID_STRING_PASSWORD_STRING
                || config.referencesSystemCredential() 
                || id.indexOf('|') != -1)
                continue;

            UserPasswordPassiveCredential cred = null;

            try {
                cred =
                    (UserPasswordPassiveCredential)cvs.getCredential(
                        id,
                        "UserPasswordPassive",
                        new Hashtable(),
                        request);
            } catch (CredentialSecretNotSetException e) {
                cred = null;
                if (getPortletLog().isWarnEnabled())
                    getPortletLog().warn(
                        "Credential secret is not set yet: "
                            + " slotId = "
                            + id);
            } catch (Exception e) {
                if (getPortletLog().isWarnEnabled())
                    getPortletLog().warn(
                        "getCredential() failed: "
                            + " slotId = "
                            + id
                            + "\n"
                            + e);
                continue;
            }

            Properties p = new Properties();
            p.setProperty("slotId", id);
            p.setProperty("resourceName", resourceName);
            p.setProperty("segmentName", segmentName);
            p.setProperty("userId", cred == null ? "" : cred.getUserId());
            p.setProperty(
                "password",
                cred == null ? "" : new String(cred.getPassword()));

                adminCredentials.add(p);
        }

        viewBean.setAccessibleUserIdPasswordCredentialsInAdminSlots(
            adminCredentials);
        return;
    }
}
