/*
        This sample code is licensed under the
        International License Agreement for
        non-warranted IBM Software and is
        provided "as-is" with no formal support.

        IBM COPYRIGHT INFORMATION:
        __________________________

        International Business Machines Corporation 1997, 2005
        Licensed program - Property of IBM . All rights reserved.
        IBM and WebSphere are trademarks of International Business
        Machines Corporation in the United States and/or other countries.
        Java is a trademark of Sun Microsystems, Incorporated.
        U.S. Government Users Restricted Rights - Use, duplication, or
        disclosure restricted by GSA ADP. Schedule Contract with IBM Corp.

        LIMIT OF LIABILITY:
        ___________________

        License to the contained APAR fixes is conditioned upon
        the recipient holding a license to the prerequisite
        product.  The license terms and conditions of the APAR
        fixes are the same as those for the prerequisite product.
        Please see the product license for specifics.
*/
package com.ibm.wps.portlets.hodwps5;

import java.io.IOException;
import java.security.*;
import java.util.*;

import javax.security.auth.Subject;

import org.apache.jetspeed.portlet.*;

import com.ibm.wps.portletservice.credentialvault.*;
import com.ibm.wps.portletservice.credentialvault.credentials.*;


/**
 *
 * A common class to handle user credentials.
 *
 */
public class PasswordEditorPortletSecretManager {

	public static final int SECRET_PORTLET_PRIVATE_SLOT	= 0;
	public static final int SECRET_SHARED_SLOT			= 1;
	public static final int SECRET_ADMINISTRATIVE_SLOT	= 2;
	public static final int SECRET_SYSTEM_SLOT			= 3;
	public static final int SECRET_JAAS_SUBJECT			= 4;

	private static CredentialVaultService vaultService = null;
	
	/**
	 * Initialize data members. "vaultService" is needed to create slot
	 * 
	 * @param config PortletConfig
	 * @throws PortletException
	 */
   	public static void init(PortletConfig config) throws PortletException {
		if( vaultService==null )
			vaultService = (CredentialVaultService)config.getContext().getService(CredentialVaultService.class);
   	}

	/**
	 * Check if UserID/Password can be saved
	 * 
	 * @param sessionBean PasswordEditorPortletSessionBean
	 * @return true if writable. false if read-only.
	 */
	public static boolean isWritable(PasswordEditorPortletSessionBean sessionBean) {
		switch(sessionBean.getSecretType()) {
			case SECRET_PORTLET_PRIVATE_SLOT:
			case SECRET_SHARED_SLOT:
			case SECRET_ADMINISTRATIVE_SLOT:
				return true;
		}
		return false;
	}

	/**
	 * Return userid and password via credential vault
	 * 
	 * @param portletRequest PortletRequest
	 * @param sessionBean PasswordEditorPortletSessionBean
	 * @param userid StringBuffer to retrieve user id
	 * @param password StringBuffer to retrieve password
	 * @throws PortletException
	 */
	public static void getCredential(PortletRequest portletRequest,PasswordEditorPortletSessionBean sessionBean,StringBuffer userid, StringBuffer password) throws PortletException {
		if( SECRET_JAAS_SUBJECT == sessionBean.getSecretType() ) {
			Subject subject = vaultService.getUserSubject(portletRequest);
			if( subject != null ) {
				userid.append(getPrincipalFromSubject(subject,"com.ibm.wps.sso.UserIdPrincipal"));
				password.append(getPrincipalFromSubject(subject,"com.ibm.wps.sso.PasswordCredential"));
			}
		} else {
			String slotId = getSlotId(portletRequest,sessionBean,false);
			if( slotId != null ) {
				UserPasswordPassiveCredential credential = (UserPasswordPassiveCredential)vaultService.getCredential(slotId,"UserPasswordPassive",new HashMap(),portletRequest);
				if( credential != null) {
					userid.append(credential.getUserId());
					password.append(String.valueOf(credential.getPassword()));
				}
			}
		}
   	}

	/**
	 * Save UserID/Password into the credential slot
	 * 
	 * @param portletRequest PortletRequest
	 * @param sessionBean PasswordEditorPortletSessionBean
	 * @param userID user id
	 * @param password oassword
	 * @return true if success
	 * @throws PortletException
	 */
	public static boolean setCredential(PortletRequest portletRequest,PasswordEditorPortletSessionBean sessionBean,String userID,String password)  throws PortletException {
		if( isWritable(sessionBean) ) {
			String slotId = getSlotId(portletRequest,sessionBean,true); // create slot if necessary
			if( slotId != null ) {
				vaultService.setCredentialSecretUserPassword(slotId,userID,password.toCharArray(),portletRequest);
				return true;
			}
		}
		return false;
	}

    /**
     * Save UserID/Password into the credential slot
     * 
     * @param portletRequest PortletRequest
     * @param slotId slot id
     * @param userID user id
     * @param password oassword
     * @return true if success
     * @throws PortletException
     */
    public static boolean setCredential(PortletRequest portletRequest, String slotId, String userID,String password)  throws PortletException {
        if( slotId != null ) {
            vaultService.setCredentialSecretUserPassword(slotId,userID,password.toCharArray(),portletRequest);
            return true;
        }
        return false;
    }

//////// Private Methods ////////

	/**
	 * Return slotId
	 * 
	 * @param portletRequest portletRequest
	 * @param sessionBean PasswordEditorPortletSessionBean
	 * @param bCreate create new slot if slot does not defined
	 * @return Slot Id
	 * @throws PortletException
	 */
   	private static String getSlotId(PortletRequest portletRequest, PasswordEditorPortletSessionBean sessionBean, boolean bCreate)  throws PortletException {
   		String slotId = null;
   		String slotName = sessionBean.getVaultSlotName();
   		switch( sessionBean.getSecretType() ) {
   			case SECRET_PORTLET_PRIVATE_SLOT:
			   	PortletData data = portletRequest.getData();
   				slotId = (String)data.getAttribute(slotName);
   				if( slotId==null && bCreate ) {
   					slotId = createNewSlot(portletRequest,slotName, true);	// create private slot
					if( slotId != null ) {
						data.setAttribute(slotName, slotId);
	   					try {
							data.store();
	   					} 
	   					catch( IOException ioe ) {
							throw(new PortletException("Error on PortletData.store()", ioe));
	   					}
					}
   				}
   				break;
   				
   			case SECRET_SHARED_SLOT:
				Iterator it = vaultService.getAccessibleSlots(portletRequest);
				while( it.hasNext() ) {
					CredentialSlotConfig config = (CredentialSlotConfig)it.next() ;
					//searches for shared resource name
		 			if( config.getResourceName().startsWith(slotName ) ) {
						slotId = config.getSlotId();
						break;
		 			}
				}
   				if( slotId==null && bCreate )
   					slotId = createNewSlot(portletRequest,slotName,false);	// create shared slot
				break;
				
   			default:
   				slotId = slotName;
   				break;
   		}
   		return slotId;
   	}

	/**
	 * Create new credential slot for the portlet
	 * 
	 * @param portletRequest PortletRequest
	 * @param slotName Slot name
	 * @param bPrivate private slot
	 * @return Slot Id
	 * @throws PortletException
	 */
	private static String createNewSlot(PortletRequest portletRequest, String slotName, boolean bPrivate)  throws PortletException  {
		CredentialSlotConfig config = vaultService.createSlot(slotName, vaultService.getDefaultUserVaultSegmentId(), new Hashtable(), new Hashtable(),
																		CredentialVaultService.SECRET_TYPE_USERID_STRING_PASSWORD_STRING, false, bPrivate, portletRequest);
		return config==null ? null : config.getSlotId();
	}

	/**
	 * Retrieves the specified Principal from the provided Subject
	 * 
	 * @param subject Subject
	 * @param className name of the princepals
	 * @return princepal
	 * @throws PortletException
	 */
	private static String getPrincipalFromSubject(Subject subject, String className) throws PortletException {
		try {
			// Get the Set of Principals
			Object[] principals = subject.getPrincipals(Class.forName(className)).toArray();
			// Do we have any?
			return (principals!=null && principals.length>0) ? ((Principal)principals[0]).getName() : "";
		} catch( ClassNotFoundException cnfe ) {
			throw( new PortletException("Class " + className + " could not be found",cnfe) );
		}
	}
}
